{ ##################################################
  # Component: TCreepingLine
  # Created:   5.11.06
  # Version:   1
  # Description:    " "
  # Copyright (c) 2006 by Alexey Savin
  ##################################################}

unit CreepingLine;

interface

uses
  Windows, Classes, Controls,Graphics, SysUtils, ExtCtrls, AlxCommon;

type
  TDirection = (dLeftToRight, dRightToLeft);
  TCreepingLine = class (TGraphicControl)
  private  { Private declarations }
    //
    FAlignment: TAlignment;
    FCaption: String;
    FActive: Boolean;
    FColor: TColor;
    FFont: TFont;
    FInterval: Cardinal;
    FFrameSides: TSides;
    FBorderSides: TSides;
    FBorderInner: TFrameStyle;
    FBorderOuter: TFrameStyle;
    FDirection: TDirection;
    // 
    ImgText: TBitmap;
    Timer1: TTimer;
    ColS, ColS1: Integer; // 
    DrawRct: TRect;
    flgScroll: Byte;
    procedure LoopTimer(Sender: TObject);
    procedure DrawLeftToRight;
    procedure DrawRightToLeft;
  protected
    procedure Paint; override;
    procedure SetAlignment(NewValue: TAlignment); virtual;
    procedure SetCaption(NewValue: String); virtual;
    procedure SetActive(NewValue: Boolean); virtual;
    procedure SetColor(NewValue: TColor); virtual;
    procedure SetFont(NewValue: TFont); virtual;
    procedure SetInterval(NewValue: Cardinal); virtual;
    procedure SetFrameSides( Value: TSides ); virtual;
    procedure SetBorderSides( Value: TSides ); virtual;
    procedure SetBorderInner( Value: TFrameStyle ); virtual;
    procedure SetBorderOuter( Value: TFrameStyle ); virtual;
    procedure SetDirection(Value: TDirection); virtual;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Align;
    property Visible;
    property ShowHint;
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property Caption: String read FCaption write SetCaption;
    property Active: Boolean read FActive write SetActive;
    property Color: TColor read FColor write SetColor;
    property Font: TFont read FFont write SetFont;
    property Interval: Cardinal read FInterval write SetInterval;
    property FrameSides: TSides read FFrameSides write SetFrameSides;
    property BorderSides: TSides read FBorderSides write SetBorderSides;
    property BorderInner: TFrameStyle read FBorderInner write SetBorderInner;
    property BorderOuter: TFrameStyle read FBorderOuter write SetBorderOuter;
    property Direction: TDirection read FDirection write SetDirection;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

procedure Register;

implementation
{$R *.res}

{ TCreepingLine }

{ ***  *** }
procedure TCreepingLine.SetAlignment(NewValue: TAlignment);
begin
  if FAlignment <> NewValue then
  begin
    FAlignment := NewValue;
    Invalidate;
  end;
end;

procedure TCreepingLine.SetCaption(NewValue: String);
begin
  if FCaption <> NewValue then begin
    Active := False;
    FCaption := NewValue;
  end;
end;

procedure TCreepingLine.SetActive(NewValue: Boolean);
begin
  FActive := NewValue;
  if FActive then begin
    ImgText.Canvas.Font.Assign(FFont);
    ImgText.Width := ImgText.Canvas.TextWidth(FCaption);
    ImgText.Height := ImgText.Canvas.TextHeight('W');
    ImgText.Canvas.Brush.Color := FColor;
    ImgText.Canvas.FillRect(Rect(0,0,ImgText.Width,ImgText.Height));
    ImgText.Canvas.TextOut(0,0,FCaption);
    Timer1.Interval := FInterval;
    ColS := 1;
    flgScroll := 1;
  end;
  Timer1.Enabled := FActive;
  Invalidate;
end;

procedure TCreepingLine.SetColor(NewValue: TColor);
begin
  if FColor <> NewValue then begin
    Active := False;
    FColor := NewValue;
  end;
end;

procedure TCreepingLine.SetFont(NewValue: TFont);
begin
  if FFont <> NewValue then begin
    Active := False;
    FFont.Assign(NewValue);
  end;
end;

procedure TCreepingLine.SetInterval(NewValue: Cardinal);
begin
  if FInterval <> NewValue then begin
    Active := False;
    FInterval := NewValue;
  end;
end;

procedure TCreepingLine.SetFrameSides(Value: TSides);
begin
  if FFrameSides <> Value then
  begin
    Active := False;
    FFrameSides := Value;
    Invalidate;
  end;
end;

procedure TCreepingLine.SetBorderSides(Value: TSides);
begin
  if FBorderSides <> Value then
  begin
    Active := False;
    FBorderSides := Value;
    Invalidate;
  end;
end;

procedure TCreepingLine.SetBorderInner(Value: TFrameStyle);
begin
  if FBorderInner <> Value then
  begin
    Active := False;
    FBorderInner := Value;
    Invalidate;
  end;
end;

procedure TCreepingLine.SetBorderOuter(Value: TFrameStyle);
begin
  if FBorderOuter <> Value then
  begin
    Active := False;
    FBorderOuter := Value;
    Invalidate;
  end;
end;

procedure TCreepingLine.SetDirection(Value: TDirection);
begin
  if FDirection <> Value then
  begin
    Active := False;
    FDirection := Value;
    Invalidate;
  end;
end;
{ ****** }
procedure TCreepingLine.LoopTimer(Sender: TObject);
begin
  if FDirection = dLeftToRight then   // - 
    DrawLeftToRight else
    DrawRightToLeft;
end;

procedure TCreepingLine.DrawLeftToRight;
Var
  RS,RD: TRect;
  Indx,H: Integer;
begin
  //     
  H := ((DrawRct.Bottom-DrawRct.Top)- ImgText.Height) div 2;
  if flgScroll = 1 then // 1 -     ;
  begin
    if DrawRct.Left+ColS < DrawRct.Right then begin   //    
      RS := Rect(ImgText.Width-ColS,0,ImgText.Width,ImgText.Height); //  
      RD := Rect(DrawRct.Left,H,DrawRct.Left+ColS,H+ImgText.Height);
    end else begin    //   
      Indx := DrawRct.Right-DrawRct.Left;
      RS := Rect(ImgText.Width-ColS,0,(ImgText.Width-ColS)+Indx,ImgText.Height); //  
      RD := Rect(DrawRct.Left,H,DrawRct.Right,H+ImgText.Height);
    end;
    inherited Canvas.CopyRect(RD,ImgText.Canvas,RS);
    Inc(ColS);
    if (ColS > ImgText.Width) then
    begin
      flgScroll := 2;
      ColS := 1;
      ColS1 := 0;
    end;
  end else if flgScroll = 2 then        //2 -         
  begin
    RD := Rect(DrawRct.Left,DrawRct.Top,DrawRct.Left+ColS,DrawRct.Bottom); //   
    inherited Canvas.FillRect(RD);
    Indx := DrawRct.Right-(DrawRct.Left+ColS);
    if (Indx < ImgText.Width) then
    begin     //     
      RS := Rect(0,0,Indx,ImgText.Height); //  
      RD := Rect(DrawRct.Left+ColS,H,DrawRct.Right,H+ImgText.Height);
    end else
    begin     //   ,  
      RS := Rect(0,0,ImgText.Width,ImgText.Height);
      RD := Rect(DrawRct.Left+ColS,H,(DrawRct.Left+ColS+ImgText.Width)-1,H+ImgText.Height);
    end;
    inherited Canvas.CopyRect(RD,ImgText.Canvas,RS);
    Inc(ColS);
    if (DrawRct.Left+ColS > DrawRct.Right) then
    begin
      flgScroll := 1;
      ColS := 1;
    end;
  end;
end;

procedure TCreepingLine.DrawRightToLeft;
Var
  RS,           // 
  RD: TRect;    //   
  Indx,H: Integer;
begin
  //     
  H := ((DrawRct.Bottom-DrawRct.Top)- ImgText.Height) div 2;

  if flgScroll = 1 then // 1 -     ;
  begin
    RS := Rect(0,0,ColS,ImgText.Height); //  
    RD := Rect(DrawRct.Right-ColS,H,DrawRct.Right,H+ImgText.Height);
    inherited Canvas.CopyRect(RD,ImgText.Canvas,RS);
     //     ,  
{    if (ImgText.Width-ColS < DrawRct.Right-DrawRct.Left) then //(DrawRct.Right-DrawRct.Left > ImgText.Width) or
    begin
      RD := Rect(DrawRct.Right-ColS,DrawRct.Top,DrawRct.Right,DrawRct.Bottom); //   
      inherited Canvas.FillRect(RD);
    end;}

    Inc(ColS);
    if (DrawRct.Right-ColS = DrawRct.Left) or (ColS > ImgText.Width) then
    begin
      flgScroll := 2;
      ColS := 1;
    end;
  end else if flgScroll = 2 then        //2 -         
  begin
    Indx := DrawRct.Right-(DrawRct.Right-ColS);       //    
    //     ,  
{    if (DrawRct.Right-DrawRct.Left > ImgText.Width) or (ImgText.Width-ColS < DrawRct.Right-DrawRct.Left) then
    begin
      RD := Rect(DrawRct.Right-ColS,DrawRct.Top,DrawRct.Right,DrawRct.Bottom); //   
      inherited Canvas.FillRect(RD);
    end;}

    if (DrawRct.Right-DrawRct.Left)-Indx > ImgText.Width then
    begin
      //      ,    
      RS := Rect(0,0,ImgText.Width,ImgText.Height);
      RD := Rect((DrawRct.Right-ColS)-ImgText.Width,H,DrawRct.Right-ColS,H+ImgText.Height);
    end else if ImgText.Width-ColS > DrawRct.Right-DrawRct.Left then
    begin
      //     ,  
      RS := Rect(ColS,0,ColS+(DrawRct.Right-DrawRct.Left),ImgText.Height);
      RD := Rect(DrawRct.Left,H,DrawRct.Right,H+ImgText.Height);
    end else
    begin
      //    
      if ColS1 = 0 then
      begin // ,     
        if (DrawRct.Right-DrawRct.Left) <= ImgText.Width then
          ColS1 := ColS + 1 else
          ColS1 := 1;
      end;
      RS := Rect(ColS1,0,ImgText.Width,ImgText.Height);
      RD := Rect(DrawRct.Left,H,DrawRct.Left+(RS.Right-RS.Left),H+ImgText.Height); //
    end;
    Inc(ColS);
    if ColS1 > 0 then
      Inc(ColS1);
    inherited Canvas.CopyRect(RD,ImgText.Canvas,RS);
    //     ,  
    if (RD.Right < DrawRct.Right) then //(DrawRct.Right-DrawRct.Left > ImgText.Width) or
    begin
      RD := Rect(RD.Right,DrawRct.Top,DrawRct.Right,DrawRct.Bottom); //   
      inherited Canvas.FillRect(RD);
    end;

    if  (ColS1 > ImgText.Width) then
    begin
      flgScroll := 1;
      ColS := 1;
      ColS1 := 0;
    end;
  end;
end;
{ - Protected procedure - }
procedure TCreepingLine.Paint;
Var
  W,H: Integer;
begin
  inherited Paint;
  DrawRct := Rect(0,0,Width,Height);
  with inherited Canvas do
  begin
    Brush.Color := FColor;
    Brush.Style := bsSolid;
    FillRect(DrawRct);
  end;
  { - ,     }
  if not FActive then with inherited Canvas do
  begin
    Font.Assign(FFont);
    H := ((DrawRct.Bottom-DrawRct.Top)- TextHeight(FCaption)) div 2;
    if FAlignment = taLeftJustify then begin
      W := DrawRct.Left+5;  //+   
    end else if FAlignment = taCenter then begin
      W := ((DrawRct.Right-DrawRct.Left) div 2) - (TextWidth(FCaption) div 2);
    end else begin
      W := DrawRct.Right-TextWidth(FCaption)-5; //+   
    end;
    TextOut(W,H,FCaption);
  end;
  DrawRct := DrawSides( Canvas, DrawRct, clBlack, clBlack, FFrameSides );
  { -   }
  if FBorderOuter = fsFlat then
    DrawRct := DrawBevel(Canvas, DrawRct, clBlack, clBlack, 1, FBorderSides)
  else if FBorderOuter = fsRaised then
    DrawRct := DrawBevel(Canvas, DrawRct, clBtnHighlight, clBtnShadow, 1, FBorderSides)
  else if FBorderOuter = fsLowered then
    DrawRct := DrawBevel(Canvas, DrawRct, clBtnShadow, clBtnHighlight, 1, FBorderSides);
  { -   }
  if FBorderInner = fsFlat then
    DrawRct := DrawBevel(Canvas, DrawRct, clBlack, clBlack, 1, FBorderSides)
  else if FBorderInner = fsRaised then
    DrawRct := DrawBevel(Canvas, DrawRct, clBtnHighlight, clBtnShadow, 1, FBorderSides)
  else if FBorderInner = fsLowered then
    DrawRct := DrawBevel(Canvas, DrawRct, clBtnShadow, clBtnHighlight, 1, FBorderSides);
end;
{ -Published procedure- }
constructor TCreepingLine.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ImgText := TBitmap.Create;
  FFont := TFont.Create;
  Timer1 := TTimer.Create(Self);
  Timer1.OnTimer := LoopTimer;
  Color := clBtnFace;
  Width := 50;
  Height := 25;
  FInterval := 10;
  Invalidate;
end;

destructor TCreepingLine.Destroy;
begin
  ImgText.Free;
  Timer1.Free;
  FFont.Free;
  inherited;
end;
{*******************}
procedure Register;
begin
  RegisterComponents('My VCL', [TCreepingLine]);
end;

end.
